package Counter;

use strict;
use DBI;
use GD;
use IO::File;
use Image::Size;

use vars qw/$VERSION/;

$VERSION = '0.01';
use constant DEFAULT_DIGIT => 6;

sub new
{
	my $class = shift;
	my %args  = @_;

	if(!defined $args{dsn}){
		die 'no dsn.';
	}

	# connect to DB
	my $dbh;
	eval {
		$dbh = DBI->connect($args{dsn}, $args{user}, $args{pass},
							{ RaiseError => 1, AutoCommit => 0 });
	};
	if($@){
		die "cannot connect to db. $DBI::errstr";
	}

	my $self;
	$self = bless { dbh  => $dbh,
					path => $args{path} ? $args{path} : '.',
				  }, $class;
	$self->_init;
}

sub _init
{
	my $self = shift;

	my $sql_max = qq|SELECT max(counter) FROM access_log|;
	my ($sth_max, $max_count);
	eval {
		$sth_max = $self->{dbh}->prepare($sql_max);
		$sth_max->execute;
		($max_count) = $sth_max->fetchrow_array;
	};
	if($@){
		die "cannot select max(counter). $@ ". $DBI::errstr;
	}
	$self->count($max_count);
	$self;
}

sub DESTROY
{
	my $self = shift;

	if($self->{dbh}){
		$self->{dbh}->commit;
		$self->{dbh}->disconnect;
	}
}

sub dbh
{
	my $self = shift;
	$self->{dbh};
}

sub increase
{
	my $self = shift;
	my %args = @_;

	$args{remote_addr} ||= $ENV{REMOTE_ADDR};
	$args{remote_host} ||= $ENV{REMOTE_HOST};
	$args{user_agent}  ||= $ENV{HTTP_USER_AGENT};
	$args{referrer}    ||= $ENV{HTTP_REFERRER};

	my $max_count = $self->count;

	my $sql = qq|INSERT INTO access_log
				 (counter, access_date, remote_addr, remote_host, user_agent, referrer)
				 VALUES (?, ?, ?, ?, ?, ?)|;
	my $sth;
	eval {
		$sth = $self->{dbh}->prepare($sql);
		$sth->execute(++$max_count,
					  $self->_get_current_timestamp,
					  $args{remote_addr},
					  $args{remote_host},
					  $args{user_agent},
					  $args{referrer});
	};
	if($@){
		die "cannot insert into access_log. $@ ". $DBI::errstr;
	}
	$self->count($max_count);
	$max_count;
}

sub count
{
	my $self = shift;
	if(@_){
		$self->{count} = shift;
	}
	$self->{count};
}

sub _get_current_timestamp
{
	my $self = shift;

	my ($sec, $min, $hour, $day, $mon, $year) = localtime;
	$year += 1900;
	$mon ++;

	sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year, $mon, $day, $hour, $min, $sec);
}

sub image
{
	my $self = shift;
	my %args = @_;
	my ($w, $h);
	my $digit = $args{digit} ? $args{digit} : DEFAULT_DIGIT;

	my $path = $self->{path};
	($w, $h) = imgsize("$path/0.png");
	if(!defined $w or !defined $h){
		die "cannot get imgsize.";
	}

	my $new_image = new GD::Image($w * $digit, $h);
	my $current_digit = 0;

	foreach my $num (split('', sprintf("%0${digit}d", $self->count))){
		my $fh = new IO::File "$path/$num.png", "r";
		if(defined $fh){
			my $num_image = newFromPng GD::Image($fh);
		    $new_image->copy($num_image, $current_digit * $w, 0, 0, 0, $w, $h);
		    $current_digit++;
		}else{
			die "cannot open image. ($path/$num.png)";
		}	
	}
	$new_image->png;
}


# Preloaded methods go here.

1;
__END__
# Below is stub documentation for your module. You better edit it!

=head1 NAME

Counter - Web Access Counter module.

=head1 SYNOPSIS

  use Counter;
  
  my $c = Counter->new( dsn => 'dbi:mysql:database=hoge',
                        user => 'foo',           # DB user
                        pass => 'bar',           # DB password
                        path => '/foo/bar/path'  # [0-9].png file path
                      );
  
  $c->increase;    # 󥿥åסϿ;
  
  if($mode eq 'text'){
      print "Content-Type: text/plain\n\n";
	  $c->count;
  }
  if($mode eq 'image'){
      print "Content-Type: image/png\n\n";
      print $c->image;
  }

=head1 DESCRIPTION

ůʤ󥿡


=head2 EXPORT

None.

=head1 method

=item B<new>

Counter ֥Ȥޤ

   : dsn  (DBI->connect 1)
         user (DBI->connect 2)
         pass (DBI->connect 3)
         path ( [0-9].png եؤΥѥǥեȤǤϥȥǥ쥯ȥ)


=item B<increase>

󥿤1äϿޤ
øΥ󥿤֤ͤޤ

Ͽơ֥ϰʲ SQL Ǻ(PostgreSQL, MySQL, SQLite)

  CREATE TABLE access_log(
      counter int,
      access_date datetime,
      remote_addr text,
      remote_host text,
      user_agent text,
      referrer text);

 remote_addr, remote_host, user_agent, referrer ꤹȡͤϿޤ
ϤʤС줾Ķѿ REMOTE_ADDR, REMOTE_HOST, HTTP_USER_AGENT, HTTP_REFERRER Ͽޤ

=item B<image>

󥿤 PNG (Хʥ)֤ޤ

󥹥ȥ饯ΰ path Ϳǥ쥯ȥ
[0-9].png Ȥե뤬¸ߤɬפޤ
ޤβեϤ٤Ʊ礭 (⤵) ǤʤƤϤʤޤ
礭㤦硢 0.png 礭ƱǤȤưޤ

 digit ꤹȡꤷβ֤ޤ(ǥեȤ6)

  $img = $c->image;
  
  $img = $c->image(digit => 10);

=item B<count>

ߤΥ󥿤֤ͤޤ

=head1 AUTHOR

ƣӰϺ (FUJIWARA Shunichiro) <fujis@gol.com>

=head1 SEE ALSO

  GD
  Image::Size
  DBI

=cut
